/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.application;

import org.mortbay.jetty.Server;
import org.mortbay.jetty.Handler;
import org.mortbay.jetty.handler.ContextHandler;
import org.mortbay.jetty.handler.ResourceHandler;
import org.mortbay.jetty.nio.SelectChannelConnector;
import org.mortbay.jetty.webapp.WebAppContext;
import org.mortbay.thread.BoundedThreadPool;
import org.mortbay.xml.XmlConfiguration;

import java.net.URISyntaxException;
import java.net.URL;

public class JettyEmbedder {
  private Server server;
  private int listenPort;

  public JettyEmbedder(int port, String contextPath, URL configurationURL) throws Exception {
    listenPort = port;
    server = new Server();
    
    if (configurationURL==null) {
      configureProgrammatically(server, port, contextPath);

    } else {
      System.setProperty("jetty.port", String.valueOf(port));
      System.setProperty("jetty.context", contextPath);
      XmlConfiguration configuration = new XmlConfiguration(configurationURL);
      configuration.configure(server);
    }
  }

  public void start() throws Exception {
    server.start();

    Runtime.getRuntime().addShutdownHook(new Thread() {
      public void run() {
        try {
          server.stop();
        } catch (Exception e) {
          System.err.println("Error occured during jetty shutdown : " + e.getMessage());
        }
      }
    });
    System.out.println("Jetty server started on port " + listenPort);
  }

  private Server configureProgrammatically(Server server, int port, String contextPath) throws URISyntaxException {
    BoundedThreadPool threadPool = new BoundedThreadPool();
    threadPool.setMinThreads(5);
    threadPool.setMaxThreads(50);
    threadPool.setLowThreads(10);
    server.setThreadPool(threadPool);
    SelectChannelConnector connector = new SelectChannelConnector();
    connector.setHost("0.0.0.0");
    connector.setPort(port);
    connector.setStatsOn(false);
    connector.setAcceptors(2);
    connector.setConfidentialPort(8443);
    server.addConnector(connector);
    server.setStopAtShutdown(true);
    server.setSendServerVersion(false);
    server.setSendDateHeader(true);
    server.setGracefulShutdown(1000);

    server.addHandler(getResourceHandler("/maven", "/war/sonar-web/public/maven", "Public,max-age=604800"));
    server.addHandler(getResourceHandler("/images", "/war/sonar-web/public/images", "Public,max-age=604800"));
    server.addHandler(getResourceHandler("/stylesheets", "/war/sonar-web/public/stylesheets", "Public,max-age=7200"));
    server.addHandler(getResourceHandler("/javascripts", "/war/sonar-web/public/javascripts", "Public,max-age=7200"));
    server.addHandler(new WebAppContext(getPath("/war/sonar-web"), contextPath));

    return server;
  }


  private Handler getResourceHandler(String context, String resourcePath, String cacheControl) throws URISyntaxException {
    String absoluteResourcePath = getPath(resourcePath);
    ContextHandler contextHandler = new ContextHandler(context);
    contextHandler.setResourceBase(absoluteResourcePath);
    ResourceHandler resourceHandler = new ResourceHandler();
    resourceHandler.setCacheControl(cacheControl);
    contextHandler.addHandler(resourceHandler);
    return contextHandler;
  }

  private String getPath(String resourcePath) throws URISyntaxException {
    URL resource = getClass().getResource(resourcePath);
    if (resource != null) {
      return resource.toURI().toString();
    }
    return null;
  }

  protected Server getServer() {
    return server;
  }

  public int getListenPort() {
    return listenPort;
  }
}